'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _babylon = require('babylon');

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _url = require('url');

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

const babylonOptions = {
	sourceType: 'module',
	plugins: ['jsx', 'asyncFunctions', 'classConstructorCall', 'doExpressions', 'trailingFunctionCommas', 'objectRestSpread', 'decorators', 'classProperties', 'exportExtensions', 'exponentiationOperator', 'asyncGenerators', 'functionBind', 'functionSent']
};

const resolvePattern = (styleURL, file) => {
	const name = styleURL.hostname === 'pattern' ? 'Pattern' : styleURL.hostname;
	if (name === 'Pattern') {
		return file.pattern;
	}
	if (name in file.dependencies) {
		return file.dependencies[name].pattern;
	}
	const error = new Error([`Could not resolve dependency ${ name }`, `in ${ file.pattern.id }:${ file.name },`, `it is not in pattern.json.`, 'Available pattern dependencies:', Object.keys(file.dependencies).join(', ')].join(' '));
	error.fileName = file.path;
	error.file = file.path;
	throw error;
};

const isStyleImport = source => {
	const protocol = (0, _url.parse)(source).protocol;
	return typeof protocol === 'string' && protocol.startsWith('style');
};
const isStaticRequire = node => node.callee.name === 'require' && t.isLiteral(node.arguments[0]);

const replaceImportDeclaration = (path, tokens) => {
	path.replaceWith(t.variableDeclaration('const', path.node.specifiers.map(specifier => {
		const value = t.isImportDefaultSpecifier(specifier) ? tokens : tokens[specifier.imported.name];

		return t.variableDeclarator(specifier.local, t.valueToNode(value));
	})));
};

const getStyleImports = ast => {
	const styleImports = [];
	(0, _babelTraverse2.default)(ast, {
		ImportDeclaration(path) {
			if (isStyleImport(path.node.source.value)) {
				styleImports.push((0, _url.parse)(path.node.source.value));
			}
		},
		CallExpression(path) {
			if (isStaticRequire(path.node) && isStyleImport(path.node.arguments[0].value)) {
				styleImports.push((0, _url.parse)(path.node.arguments[0]));
			}
		}
	});
	return styleImports;
};

const replaceStyleImports = (ast, tokensByIdentifier) => {
	(0, _babelTraverse2.default)(ast, {
		ImportDeclaration(path) {
			if (isStyleImport(path.node.source.value)) {
				const tokens = tokensByIdentifier[path.node.source.value.toLowerCase()];
				replaceImportDeclaration(path, tokens);
			}
		},
		CallExpression(path) {
			if (isStaticRequire(path.node) && isStyleImport(path.node.arguments[0].value)) {
				const tokens = tokensByIdentifier[path.node.arguments[0].toLowerCase()];
				path.replaceWith(t.valueToNode(tokens));
			}
		}
	});
};

const getStyleBaseName = (styleImport, pattern) => {
	var _styleImport$protocol = styleImport.protocol.slice(0, -1).split('+');

	var _styleImport$protocol2 = _slicedToArray(_styleImport$protocol, 2);

	const format = _styleImport$protocol2[0];
	var _styleImport$protocol3 = _styleImport$protocol2[1];
	const concern = _styleImport$protocol3 === undefined ? 'index' : _styleImport$protocol3;

	const outFormat = (0, _lodash.find)(pattern.outFormats, outFormat => outFormat.type === format);
	if (!outFormat) {
		const error = new Error(`Pattern ${ pattern.id } has no file matching format: ${ format }`);
		error.fileName = pattern.path;
		error.file = pattern.path;
		throw error;
	}
	return `${ concern }.${ outFormat.extension }`;
};

const getStyleTokens = (() => {
	var ref = _asyncToGenerator(function* (styleImports, file, application) {
		return yield Promise.all(styleImports.map((() => {
			var ref = _asyncToGenerator(function* (styleImport) {
				const pattern = resolvePattern(styleImport, file);
				const fileName = getStyleBaseName(styleImport, pattern);
				const stylePattern = yield application.pattern.factory(pattern.id, pattern.base, {
					patterns: application.configuration.patterns,
					transforms: application.configuration.transforms,
					log: application.log
				}, application.transforms, {
					outFormats: [_path2.default.extname(fileName).slice(1)],
					baseNames: [_path2.default.basename(fileName, _path2.default.extname(fileName))]
				});
				yield stylePattern.read();
				yield stylePattern.transform();

				const styleFile = stylePattern.files[fileName];

				if (!styleFile) {
					const error = new Error([`Pattern ${ stylePattern.id } has no file ${ fileName }`, `requested by "${ styleImport.href }" in ${ file.pattern.id }:${ file.name }.`, `Available files: ${ Object.keys(stylePattern.files).join(', ') }`].join(' '));
					error.fileName = pattern.path;
					error.file = pattern.path;
					throw error;
				}

				const tokens = styleFile.meta.cssmodules;

				if (!tokens) {
					const error = new Error(['Could not find cssmodules meta data for', `${ styleFile.pattern.id }:${ styleFile.baseName }`, `imported by ${ file.pattern.id }:${ file.baseName }.`, ' Did you forget to turn on cssmodules transform?'].join(''));

					error.fileName = file.path;
					error.file = file.path;
				}

				return { styleImport, tokens };
			});

			return function (_x4) {
				return ref.apply(this, arguments);
			};
		})()));
	});

	return function getStyleTokens(_x, _x2, _x3) {
		return ref.apply(this, arguments);
	};
})();

const transform = (() => {
	var ref = _asyncToGenerator(function* (application, file) {
		const transformingDependencies = Object.values(file.dependencies).map(function (file) {
			return transform(application, file);
		});

		const source = file.buffer.toString('utf-8');
		const ast = (0, _babylon.parse)(source, babylonOptions);

		const styleImports = getStyleImports(ast);
		const styleTokens = yield getStyleTokens(styleImports, file, application);

		replaceStyleImports(ast, styleTokens.reduce(function (tokensByIdentifier, tokens) {
			tokensByIdentifier[tokens.styleImport.href] = tokens.tokens;
			return tokensByIdentifier;
		}, {}));

		file.buffer = (0, _babelGenerator2.default)(ast.program).code;

		/**
   * For "plain" jsx files we are dealing with invalid js
   * because babel-generator appends a trailing semi-colon
   * to the jsx expression, causing subsequent babel parse
   * operations to fail, thus we have to remove it.
   * TODO: Remove this when when
   * - soft-deprecated plain jsx without default export
   * - hard-deprecated plain jsx without default export
   */
		if (file.buffer[file.buffer.length - 1] === ';') {
			file.buffer = file.buffer.slice(0, file.buffer.length - 1);
		}

		yield Promise.all(transformingDependencies);
		return file;
	});

	return function transform(_x5, _x6) {
		return ref.apply(this, arguments);
	};
})();

exports.default = application => {
	return (() => {
		var ref = _asyncToGenerator(function* (file) {
			return transform(application, file);
		});

		return function (_x7) {
			return ref.apply(this, arguments);
		};
	})();
};

module.exports = exports['default'];